/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLEditor.java,v 1.9 2007/04/16 07:54:23 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.StringUtils;
import 
    org.objectweb.petals.tools.plugin.development.jbieditor.markers.MarkingErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.LocatorImpl;

public class XMLEditor extends TextEditor {
    
    private static final String VALIDATION_FAILED = Messages.XMLEditor_0;

    private ColorManager colorManager;

    private IEditorInput input;

    private Preferences prefs;

    private String jbiXsdPath;

    /**
     * 
     */
    public XMLEditor() {
        super();
        colorManager = new ColorManager();
        setSourceViewerConfiguration(new XMLConfiguration(colorManager));
        setDocumentProvider(new XMLDocumentProvider());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.editors.text.TextEditor#dispose()
     */
    public void dispose() {
        colorManager.dispose();
        super.dispose();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.editors.text.TextEditor#doSetInput(org.eclipse.ui.IEditorInput)
     */
    protected void doSetInput(IEditorInput newInput) throws CoreException {
        super.doSetInput(newInput);
        this.input = newInput;
        prefs = Activator.getDefault().getPluginPreferences();
        jbiXsdPath = prefs
                .getString(ESBDevPrefConstants.JBI_XSD_PATH_NAME);
        if (StringUtils.isNotEmpty(jbiXsdPath)) {
            validateAndMark();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.texteditor.AbstractTextEditor#editorSaved()
     */
    protected void editorSaved() {
        super.editorSaved();
        // we validate and mark document here
        prefs = Activator.getDefault().getPluginPreferences();
        jbiXsdPath = prefs
                .getString(ESBDevPrefConstants.JBI_XSD_PATH_NAME);
        if (StringUtils.isNotEmpty(jbiXsdPath)) {
            validateAndMark();
        }
    }

    /**
     * Does the XSD validation for the edited XML file
     */
    public void validateAndMark() {
        try {
            IDocument document = getInputDocument();
            InputStream textStream = new ByteArrayInputStream(document.get()
                    .getBytes());

            MarkingErrorHandler markingErrorHandler = new MarkingErrorHandler(
                    getInputFile(), document);
            markingErrorHandler.setDocumentLocator(new LocatorImpl());
            markingErrorHandler.removeExistingMarkers();

            // XSD validation with Validator class does not working
            // Line and column numbers are equals to "-1"
            // DocumentBuilderFactory fabriqueAnalyseur =
            // DocumentBuilderFactory.newInstance();
            // DocumentBuilder analyseur =
            // fabriqueAnalyseur.newDocumentBuilder();
            // Document document2 = analyseur.parse(textStream);
            //
            // SchemaFactory fabrique = SchemaFactory.newInstance(
            // XMLConstants.W3C_XML_SCHEMA_NS_URI);
            // fabrique.setErrorHandler(markingErrorHandler);
            //
            // InputSource sourceentree = new InputSource("my_path/jbi.xsd");
            // SAXSource sourceXSD = new SAXSource(sourceentree);
            //
            // Schema schema = fabrique.newSchema(sourceXSD);
            // Validator validateur = schema.newValidator();
            //
            // validateur.validate(new DOMSource(document2));

            // So we use validation with DocumentBuilderFactory correctly
            // configured
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setAttribute(
                    "http://java.sun.com/xml/jaxp/properties/schemaLanguage", //$NON-NLS-1$
                    XMLConstants.W3C_XML_SCHEMA_NS_URI);

            File jbiXsdFile = new File(jbiXsdPath);

            dbf.setAttribute(
                    "http://java.sun.com/xml/jaxp/properties/schemaSource", //$NON-NLS-1$
                    jbiXsdFile);

            dbf.setNamespaceAware(true);
            dbf.setValidating(true);

            DocumentBuilder constructeur = dbf.newDocumentBuilder();
            constructeur.setErrorHandler(markingErrorHandler);
            constructeur.parse(textStream);
        } catch (SAXException e) {
            MessageDialog.openError(getSite().getShell(), JbiEditor.JBI_EDITOR,
                    VALIDATION_FAILED);
        } catch (ParserConfigurationException e) {
            MessageDialog.openError(getSite().getShell(), JbiEditor.JBI_EDITOR,
                    VALIDATION_FAILED);
        } catch (IOException e) {
            MessageDialog.openError(getSite().getShell(), JbiEditor.JBI_EDITOR,
                    VALIDATION_FAILED);
        }
    }

    /**
     * @return
     */
    protected IDocument getInputDocument() {
        IDocument document = getDocumentProvider().getDocument(input);
        return document;
    }

    /**
     * @return
     */
    protected IFile getInputFile() {
        IFileEditorInput ife = (IFileEditorInput) input;
        IFile file = ife.getFile();
        return file;
    }

    /**
     * @return The Editor Input
     */
    public IEditorInput getInput() {
        return input;
    }

}
