/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: TagContentAssistProcessor.java,v 1.4 2007/04/17 09:49:35 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.contentassist;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.Token;
import org.objectweb.petals.tools.plugin.development.constants.JbiXmlTags;
import org.objectweb.petals.tools.plugin.development.jbieditor.scanners.XMLTagScanner;
import org.objectweb.petals.tools.plugin.development.jbieditor.xml.XMLElement;
import org.objectweb.petals.tools.plugin.development.jbieditor.xml.XMLTree;

/**
 * Based on a tutorial wich can be found here :
 * http://www.realsolve.co.uk/site/tech/jface-text.php
 * 
 * @author Phil Zoio
 */
public class TagContentAssistProcessor implements IContentAssistProcessor {

    private XMLTree dtdTree = null;

    private XMLTagScanner scanner;

    public TagContentAssistProcessor(XMLTagScanner scanner) {
        super();
        this.dtdTree = new XMLTree();
        this.scanner = scanner;
    }

    public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer,
            int offset) {

        IDocument document = viewer.getDocument();
        boolean isAttribute = isAttribute(offset, document);

        TextInfo currentText = currentText(document, offset);

        if (!isAttribute) {
            List allElements = dtdTree.getAllElements();

            ICompletionProposal[] result = new ICompletionProposal[allElements
                    .size()];
            int i = 0;
            for (Iterator iter = allElements.iterator(); iter.hasNext();) {
                XMLElement element = (XMLElement) iter.next();
                String name = element.getName();

                String text = null;

                if (useContractedElementCompletion(offset, document)) {
                    text = name;
                } else {
                    text = "" + name + JbiXmlTags.TAG_END //$NON-NLS-1$
                        + JbiXmlTags.TAG_CLOSE_START + name + JbiXmlTags.TAG_END;
                }

                result[i++] = new CompletionProposal(text,
                        currentText.documentOffset, currentText.text.length(),
                        text.length());

            }
            return result;

        } else {

            List allAttributes = dtdTree.getAllAttributes();

            ICompletionProposal[] result = new ICompletionProposal[allAttributes
                    .size()];
            int i = 0;
            for (Iterator iter = allAttributes.iterator(); iter.hasNext();) {
                String name = (String) iter.next();

                String text = null;

                if (currentText.isWhiteSpace) {
                    text = name + "= \"\" "; //$NON-NLS-1$
                } else {
                    text = name;
                }

                result[i++] = new CompletionProposal(text,
                        currentText.documentOffset, currentText.text.length(),
                        text.length());
            }
            return result;
        }

    }

    private TextInfo currentText(IDocument document, int documentOffset) {
        
        TextInfo textInfo = null;
        
        try {
            ITypedRegion region = document.getPartition(documentOffset);
            int partitionOffset = region.getOffset();
            int partitionLength = region.getLength();
            int index = documentOffset - partitionOffset;
            String partitionText = document.get(partitionOffset, partitionLength);
            char c = partitionText.charAt(index);

            if (Character.isWhitespace(c)
                    || Character.isWhitespace(partitionText.charAt(index - 1))) {
                textInfo = new TextInfo("", documentOffset, true); //$NON-NLS-1$
            } else if (c == '<') {
                textInfo = new TextInfo("", documentOffset, true); //$NON-NLS-1$
            } else {
                int start = index;
                c = partitionText.charAt(start);

                while (!Character.isWhitespace(c) && c != '<' && start >= 0) {
                    start--;
                    c = partitionText.charAt(start);
                }
                start++;

                int end = index;
                c = partitionText.charAt(end);

                while (!Character.isWhitespace(c) && c != '>'
                        && end < partitionLength - 1) {
                    end++;
                    c = partitionText.charAt(end);
                }

                String substring = partitionText.substring(start, end);
                textInfo = new TextInfo(substring, partitionOffset + start, false);
            }
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
        
        return textInfo;
    }

    /**
     * Used to determine whether the current offset is an attribute. Will return
     * true if it finds any [text][whitespace][text] pattern between the within
     * the current partition
     */
    private boolean isAttribute(int documentOffset, IDocument document) {

        boolean isAttribute = false;

        try {
            ITypedRegion region = document.getPartition(documentOffset);

            int partitionOffset = region.getOffset();

            int readLength = documentOffset - partitionOffset;
            
            scanner.setRange(document, partitionOffset, readLength);

            boolean textReached = false;

            IToken token = null;
            while ((token = scanner.nextToken()) != Token.EOF) {
                if (token.getData() instanceof TextAttribute) {
                    textReached = true;
                    continue;
                }

                if (textReached && token.isWhitespace()) {
                    isAttribute = true;
                }

            }

        } catch (BadLocationException e) {
            e.printStackTrace();
        }
        return isAttribute;
    }

    /**
     * Used to determine whether there is any text after the current offset
     * within the same partition, excluding the current word Also returns true
     * if there is no white
     */
    private boolean useContractedElementCompletion(int documentOffset,
            IDocument document) {

        boolean textReached = false;
        boolean isRemainingWhiteSpace = true;

        try {
            ITypedRegion region = document.getPartition(documentOffset);

            int partitionOffset = region.getOffset();
            int partitionLength = region.getLength();

            int readLength = documentOffset - partitionOffset;
            int remainingLength = partitionLength - readLength;

            if (document.getLength() >= documentOffset + 1) {
                String firstTwo = document.get(partitionOffset, 2);
                if (firstTwo.equals("<<")) { //$NON-NLS-1$
                    return false;
                }
            }

            scanner.setRange(document, documentOffset, remainingLength);

            IToken token = null;
            
            do {
                token = scanner.nextToken();
                isRemainingWhiteSpace = false;
                continue;
            } while ((token != Token.WHITESPACE) && (token != Token.EOF));

            do {
                token = scanner.nextToken();
                isRemainingWhiteSpace = true;
                continue;
            }
            while ((token == Token.WHITESPACE) && (token != Token.EOF));

            char c = (char) 0;

            while (c == scanner.read()) {
                if (c == XMLTagScanner.EOF) {
                    break;
                }
                if (c == '<') {
                    break;
                }
                if (!Character.isWhitespace(c)) {
                    textReached = true;
                }
            }

        } catch (BadLocationException e) {
            e.printStackTrace();
        }

        if (textReached) {
            return true;
        }
        
        return !isRemainingWhiteSpace && !textReached;
    }

    public IContextInformation[] computeContextInformation(ITextViewer viewer,
            int offset) {
        return null;
    }

    public char[] getCompletionProposalAutoActivationCharacters() {
        return null;
    }

    public char[] getContextInformationAutoActivationCharacters() {
        return null;
    }

    public String getErrorMessage() {
        return null;
    }

    public IContextInformationValidator getContextInformationValidator() {
        return null;
    }

    static class TextInfo {
        
        private String text;
        private boolean isWhiteSpace;
        private int documentOffset;
        
        TextInfo(String text, int documentOffset, boolean isWhiteSpace) {
            this.text = text;
            this.isWhiteSpace = isWhiteSpace;
            this.documentOffset = documentOffset;
        }
    }

}
