/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.maven;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.XmlParsing;
import org.objectweb.petals.tools.plugin.development.utils.DomUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * POM files utilities
 * 
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 * 
 */
public class PomUtils {

    /**
     * Get the project information from the project POM file
     * 
     * @return
     */
    public static ArtifactInformation getProjectInformation(String projectName) {
        ArtifactInformation info = null;

        IFile pomFile = getPomForProject(projectName);
        if (pomFile != null) {
            try {
                Document pomDom = XmlParsing.parse(pomFile.getContents());
                Element rootElement = pomDom.getDocumentElement();

                // get module the information
                Element artifactNode = DomUtils.getChildFromNode(rootElement,
                        PomXmlTags.ARTIFACT_ID);
                Element groupNode = DomUtils.getChildFromNode(rootElement,
                        PomXmlTags.GROUP_ID);
                Element versionNode = DomUtils.getChildFromNode(rootElement,
                        PomXmlTags.VERSION);
                Element typeNode = DomUtils.getChildFromNode(rootElement,
                        PomXmlTags.PACKAGING);

                // type is optional (If not filled, maven takes the JAR one)
                String type = null;
                if (typeNode.getTextContent() != null) {
                    type = typeNode.getTextContent();
                }

                info = new ArtifactInformation(artifactNode.getTextContent(),
                        groupNode.getTextContent(), versionNode
                                .getTextContent(), type);

            } catch (CoreException e) {
            }
        }

        return info;
    }

    /**
     * Get the component that is the target of the given service unit.
     * 
     * @param suName
     * @return the information of the component or null if there is no target
     *         component
     */
    public static final ArtifactInformation getJBIComponentTargetForSU(
            String suName) {
        ArtifactInformation info = null;

        IFile pomFile = getPomForProject(suName);
        if (pomFile != null) {
            try {
                Document pomDom = XmlParsing.parse(pomFile.getContents());
                Element rootElement = pomDom.getDocumentElement();

                // check that this project is a SU one
                Element packElement = DomUtils.getChildFromNode(rootElement,
                        PomXmlTags.PACKAGING);

                if (packElement != null
                        && packElement.getTextContent().equals(
                                Constants.SERVICE_UNIT_PACKAGING)) {

                    Element dependencies = DomUtils.getChildFromNode(
                            rootElement, PomXmlTags.DEPENDENCIES);

                    NodeList nodes = dependencies.getChildNodes();
                    boolean found = false;
                    for (int i = 0; i < nodes.getLength() && !found; i++) {
                        // node is a dependency element
                        Node n = nodes.item(i);
                        if (n instanceof Element) {
                            ArtifactInformation tmp = getInformation((Element) n);
                            if (tmp != null
                                    && tmp.getType().equals(
                                            Constants.COMPONENT_PACKAGING)) {
                                info = tmp;
                                found = true;
                            }
                        }
                    }
                }

            } catch (CoreException e) {
            }
        }

        return info;
    }

    /**
     * Get the POM file for the given project or return null project or file not
     * found.
     * 
     * @param projectName
     * @return null if the project or the pom file has not been found
     */
    public static final IFile getPomForProject(String projectName) {
        IFile result = null;
        IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(
                projectName);

        if (project != null) {
            IFile pomFile = project.getFile(Constants.POM_PATH);
            if (pomFile.exists()) {
                result = pomFile;
            }
        }

        return result;
    }

    /**
     * Get the information from the given element. The information is retrieved
     * from the children elements.
     * 
     * @param element
     * @return null if no info has been found or if input element is null
     */
    public static final ArtifactInformation getInformation(Element element) {
        ArtifactInformation info = null;
        if (element != null) {

            Element artifactNode = DomUtils.getChildFromNode(element,
                    PomXmlTags.ARTIFACT_ID);
            Element groupNode = DomUtils.getChildFromNode(element,
                    PomXmlTags.GROUP_ID);
            Element versionNode = DomUtils.getChildFromNode(element,
                    PomXmlTags.VERSION);
            Element typeNode = DomUtils.getChildFromNode(element,
                    PomXmlTags.TYPE);

            if (artifactNode != null && groupNode != null) {
                String version = ""; //$NON-NLS-1$
                if (versionNode != null) {
                    version = versionNode.getTextContent();
                }
                String type = ""; //$NON-NLS-1$
                if (typeNode != null) {
                    type = typeNode.getTextContent();
                }

                info = new ArtifactInformation(artifactNode.getTextContent(),
                        groupNode.getTextContent(), version, type);
            }

        }
        return info;
    }

    /**
     * Creates a new instance of {@link PomUtils}
     * 
     */
    private PomUtils() {
    }
}
