/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServicesTableViewer.java,v 1.12 2007/04/25 14:51:30 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.viewers;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.objectweb.petals.tools.plugin.development.constants.WizardComposites;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SeConsumesProvides;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.ServiceTableLine;
import
    org.objectweb.petals.tools.plugin.development.providers.ServicesTableContentProvider;
import org.objectweb.petals.tools.plugin.development.providers.ServicesTableLabelProvider;
import org.objectweb.petals.tools.plugin.development.providers.ServicesTableList;

/**
 * The Services TableView based on the TableViewerExample from Laurent Gauthier.
 */
public class ServicesTableViewer {

    /**
     * Creation mode for the viewer
     */
    public static final int MODE_PROVIDES = 0;

    /**
     * Creation mode for the viewer
     */
    public static final int MODE_CONSUMES = 1;
    
    private static final int COL_SIZE = 3;
    // Set the table column property names
    private static final String INTERFACE_NAME = Messages.ServicesTableViewer_0;
    private static final String SERVICE_NAME = Messages.ServicesTableViewer_1;
    private static final String ENDPOINT_NAME = Messages.ServicesTableViewer_2;
    private static final String LINK_TYPE = Messages.ServicesTableViewer_3;
    private static final int MARGIN_WIDTH = 4;
    // Set column names
    private String[] columnNames;
    private TableViewer tableViewer;
    private Table table;
    private FormToolkit toolkit;
    // 0 : Provides, 1 : Consumes
    private int mode;
    // Create a ServicesTableList and assign it to an instance variable
    private ServicesTableList servicesTableList;
    private Button add;
    private Button delete;

    /**
     * @param parent
     * @param toolkit
     * @param mode
     */
    public ServicesTableViewer(Composite parent, FormToolkit toolkit, int mode) {
        this.mode = mode;
        this.toolkit = toolkit;

        if (mode == MODE_CONSUMES) {
            columnNames = new String[] { INTERFACE_NAME, SERVICE_NAME,
                    ENDPOINT_NAME, LINK_TYPE };
        } else {
            columnNames = new String[] { INTERFACE_NAME, SERVICE_NAME,
                    ENDPOINT_NAME };
        }

        this.addChildControls(parent);
    }

    /**
     * Create a new shell, add the widgets, open the shell
     * 
     * @return the shell that was created
     */
    private void addChildControls(Composite composite) {

        // The input for the table viewer is the instance of ServicesTableList
        servicesTableList = new ServicesTableList();

        // Create a composite to hold the children
        GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
                | GridData.FILL_BOTH);
        composite.setLayoutData(gridData);

        // Set numColumns to 3 for the buttons
        GridLayout layout = new GridLayout(COL_SIZE, false);
        layout.marginWidth = MARGIN_WIDTH;
        composite.setLayout(layout);

        // Create the table
        createTable(composite);

        // Create and setup the TableViewer
        createTableViewer();

        tableViewer.setContentProvider(new ServicesTableContentProvider(
                servicesTableList, tableViewer));
        tableViewer.setLabelProvider(new ServicesTableLabelProvider());
        tableViewer.setInput(servicesTableList);

        // Add the buttons
        createButtons(composite);
    }

    /**
     * Create the Table
     */
    private void createTable(Composite parent) {
        int style = SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

        table = toolkit.createTable(parent, style);

        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.grabExcessVerticalSpace = true;
        gridData.horizontalSpan = COL_SIZE;
        table.setLayoutData(gridData);

        table.setLinesVisible(true);
        table.setHeaderVisible(true);

        // 1st column with Interface name
        TableColumn column = new TableColumn(table, SWT.LEFT, 0);
        column.setText(columnNames[0]);
        column.setWidth(WizardComposites.COLUMN_WIDTH);

        // 2nd column with Service name
        column = new TableColumn(table, SWT.LEFT, 1);
        column.setText(columnNames[1]);
        column.setWidth(WizardComposites.COLUMN_WIDTH);

        // 3th column with Endpoint name
        column = new TableColumn(table, SWT.LEFT, 2);
        column.setText(columnNames[2]);
        column.setWidth(WizardComposites.COLUMN_WIDTH);

        if (mode == MODE_CONSUMES) {
            // 4th column with Link type
            column = new TableColumn(table, SWT.LEFT, COL_SIZE);
            column.setText(columnNames[COL_SIZE]);
            column.setWidth(WizardComposites.COLUMN_WIDTH);
        }
    }

    /**
     * Create the TableViewer
     */
    private void createTableViewer() {
        tableViewer = new TableViewer(table);
        tableViewer.setUseHashlookup(true);

        tableViewer.setColumnProperties(columnNames);

        // Create the cell editors
        CellEditor[] editors = new CellEditor[columnNames.length];

        // Column 1 : Interface name
        TextCellEditor textEditor = new TextCellEditor(table);
        ((Text) textEditor.getControl()).setTextLimit(WizardComposites.COLUMN_WIDTH);
        editors[0] = textEditor;

        // Column 2 : Service name
        textEditor = new TextCellEditor(table);
        ((Text) textEditor.getControl()).setTextLimit(WizardComposites.COLUMN_WIDTH);
        editors[1] = textEditor;

        // Column 3 : Endpoint name
        textEditor = new TextCellEditor(table);
        ((Text) textEditor.getControl()).setTextLimit(WizardComposites.COLUMN_WIDTH);
        editors[2] = textEditor;

        if (mode == MODE_CONSUMES) {
            // Column 4 : Link type
            textEditor = new TextCellEditor(table);
            ((Text) textEditor.getControl()).setTextLimit(WizardComposites.COLUMN_WIDTH);
            editors[COL_SIZE] = textEditor;
        }

        // Assign the cell editors to the viewer
        tableViewer.setCellEditors(editors);
        // Set the cell modifier for the viewer
        tableViewer.setCellModifier(new ServicesTableCellModifier(this));
    }

    /**
     * Add the "Add", "Delete" and "Close" buttons
     * 
     * @param parent the parent composite
     */
    private void createButtons(Composite parent) {
        // Create and configure the "Add" button
        add = toolkit.createButton(parent, Messages.ServicesTableViewer_4, SWT.PUSH | SWT.CENTER);
        GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
        gridData.widthHint = WizardComposites.BUTTONS_WIDTH;
        add.setLayoutData(gridData);
        add.addSelectionListener(new SelectionAdapter() {

            // Add a task to the ServicesTableList and refresh the view
            public void widgetSelected(SelectionEvent e) {
                servicesTableList.addService();
            }
        });

        // Create and configure the "Delete" button
        delete = toolkit.createButton(parent, Messages.ServicesTableViewer_5, SWT.PUSH | SWT.CENTER);
        gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
        gridData.widthHint = WizardComposites.BUTTONS_WIDTH;
        delete.setLayoutData(gridData);

        delete.addSelectionListener(new SelectionAdapter() {

            // Remove the selection and refresh the view
            public void widgetSelected(SelectionEvent e) {
                ServiceTableLine service = 
                    (ServiceTableLine) ((IStructuredSelection) tableViewer
                        .getSelection()).getFirstElement();
                if (service != null) {
                    servicesTableList.removeService(service);
                }
            }
        });
    }

    /**
     * Return the column names in a collection
     * 
     * @return List containing column names
     */
    public java.util.List getColumnNames() {
        return Arrays.asList(columnNames);
    }

    /**
     * @return currently selected item
     */
    public ISelection getSelection() {
        return tableViewer.getSelection();
    }

    /**
     * Return the ServicesTableList
     */
    public ServicesTableList getServiceList() {
        return servicesTableList;
    }

    /**
     * Return the parent composite
     */
    public Control getControl() {
        return table.getParent();
    }

    /**
     * Removes all the services from the TableViewer
     */
    public void clearServicesTable() {
        table.removeAll();
    }

    public void setInput(List<SeConsumesProvides> seList) {
        servicesTableList = new ServicesTableList(seList);
        tableViewer.setContentProvider(new ServicesTableContentProvider(
                servicesTableList, tableViewer));
        tableViewer.setLabelProvider(new ServicesTableLabelProvider());
        tableViewer.setInput(servicesTableList);
    }

    /**
     * @return the table
     */
    public Table getTable() {
        return table;
    }

    /**
     * @return the servicesTableList
     */
    public ServicesTableList getServicesTableList() {
        return servicesTableList;
    }

    /**
     * @return the add
     */
    public Button getAdd() {
        return add;
    }

    /**
     * @return the delete
     */
    public Button getDelete() {
        return delete;
    }
}
