/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SimpleTableViewer.java,v 1.7 2007/04/25 14:51:30 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.viewers;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.objectweb.petals.tools.plugin.development.constants.WizardComposites;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.TableLine;
import org.objectweb.petals.tools.plugin.development.providers.SimpleTableContentProvider;
import org.objectweb.petals.tools.plugin.development.providers.SimpleTableLabelProvider;
import org.objectweb.petals.tools.plugin.development.providers.SimpleTableList;

/**
 * A simple Table Viewer with one column
 */
public class SimpleTableViewer {

    private static final int COL_SIZE = 3;
    private static final int MARGIN_WIDTH = 4;
    
    private TableViewer tableViewer;
    private Table table;
    private FormToolkit toolkit;
    private SimpleTableList simpleTableList;
    // Set column names
    private String[] columnNames = new String[] { Messages.SimpleTableViewer_0 };
    private Button add;
    private Button delete;
    
    private SelectionAdapter addSelectionAdapter;
    private SelectionAdapter removeSelectionAdapter;

    /**
     * @param parent
     * @param toolkit
     */
    public SimpleTableViewer(Composite parent, FormToolkit toolkit) {
        this.toolkit = toolkit;
        this.addChildControls(parent);
    }
    
    /**
     * @return the add
     */
    public Button getAdd() {
        return add;
    }

    /**
     * @return the delete
     */
    public Button getDelete() {
        return delete;
    }

    /**
     * Create a new shell, add the widgets
     * 
     * @return the shell that was created
     */
    private void addChildControls(Composite composite) {
        // The input for the table viewer
        simpleTableList = new SimpleTableList();

        // Create a composite to hold the children
        GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
                | GridData.FILL_BOTH);
        composite.setLayoutData(gridData);

        // Set numColumns to 3 for the buttons
        GridLayout layout = new GridLayout(COL_SIZE, false);
        layout.marginWidth = MARGIN_WIDTH;
        composite.setLayout(layout);

        // Create the table
        createTable(composite);

        // Create and setup the TableViewer
        createTableViewer();

        tableViewer.setContentProvider(new SimpleTableContentProvider(
                simpleTableList, tableViewer));
        tableViewer.setLabelProvider(new SimpleTableLabelProvider());
        tableViewer.setInput(simpleTableList);

        // Add the buttons
        createButtons(composite);
    }

    /**
     * Create the Table
     */
    private void createTable(Composite parent) {
        int style = SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

        table = toolkit.createTable(parent, style);

        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.grabExcessVerticalSpace = true;
        gridData.horizontalSpan = COL_SIZE;
        table.setLayoutData(gridData);

        table.setLinesVisible(true);

        // 1st column
        TableColumn column = new TableColumn(table, SWT.LEFT, 0);
        column.setWidth(WizardComposites.COLUMN_WIDTH);
    }

    /**
     * Create the TableViewer
     */
    private void createTableViewer() {
        tableViewer = new TableViewer(table);
        tableViewer.setUseHashlookup(true);
        tableViewer.setColumnProperties(columnNames);

        // Create the cell editors
        CellEditor[] editors = new CellEditor[1];

        // Column 1 : Interface name
        TextCellEditor textEditor = new TextCellEditor(table);
        ((Text) textEditor.getControl()).setTextLimit(WizardComposites.COLUMN_WIDTH);
        editors[0] = textEditor;

        // Assign the cell editors to the viewer
        tableViewer.setCellEditors(editors);
        // Set the cell modifier for the viewer
        tableViewer.setCellModifier(new SimpleTableCellModifier(this));
    }

    /**
     * Add the "Add", "Delete" and "Close" buttons
     * 
     * @param parent
     *            the parent composite
     */
    private void createButtons(Composite parent) {
        // Create and configure the "Add" button
        add = toolkit.createButton(parent, Messages.SimpleTableViewer_1, SWT.PUSH | SWT.CENTER);
        GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
        gridData.widthHint = WizardComposites.BUTTONS_WIDTH;
        add.setLayoutData(gridData);
        addSelectionAdapter = new SelectionAdapter() {

            // Add a task to the ServicesTableList and refresh the view
            public void widgetSelected(SelectionEvent e) {
                simpleTableList.addLine();
            }
        };
        add.addSelectionListener(addSelectionAdapter);

        // Create and configure the "Delete" button
        delete = toolkit.createButton(parent, Messages.SimpleTableViewer_2, SWT.PUSH | SWT.CENTER);
        gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
        gridData.widthHint = WizardComposites.BUTTONS_WIDTH;
        delete.setLayoutData(gridData);

        removeSelectionAdapter = new SelectionAdapter() {
            // Remove the selection and refresh the view
            public void widgetSelected(SelectionEvent e) {
                TableLine line = (TableLine) ((IStructuredSelection) tableViewer
                        .getSelection()).getFirstElement();
                if (line != null) {
                    simpleTableList.removeLine(line);
                }
            }
        };
        delete.addSelectionListener(removeSelectionAdapter);
    }

    /**
     * Return the column names in a collection
     * 
     * @return List containing column names
     */
    public java.util.List getColumnNames() {
        return Arrays.asList(columnNames);
    }

    /**
     * @return currently selected item
     */
    public ISelection getSelection() {
        return tableViewer.getSelection();
    }

    /**
     * Return the SimpleTableList
     */
    public SimpleTableList getSimpleTableList() {
        return simpleTableList;
    }

    /**
     * Return the parent composite
     */
    public Control getControl() {
        return table.getParent();
    }

    /**
     * Removes all the services from the TableViewer
     */
    public void clearTableLines() {
        table.removeAll();
    }

    public void setInput(List<String> simpleList) {
        simpleTableList = new SimpleTableList(simpleList);
        tableViewer.setContentProvider(new SimpleTableContentProvider(
                simpleTableList, tableViewer));
        tableViewer.setLabelProvider(new SimpleTableLabelProvider());
        tableViewer.setInput(simpleTableList);
    }

    /**
     * @return the table
     */
    public Table getTable() {
        return table;
    }
    
    public void removeAddListener() {
        add.removeSelectionListener(addSelectionAdapter);
    }
    
    public void removeRemoveListener() {
        delete.removeSelectionListener(removeSelectionAdapter);
    }
}
