package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.HashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;


/**
 * In the "create new Component wizard", there is a part of the wizard about "generated artifacts".
 * This class defines the content of this part. 
 * 
 * @author Vincent Zurczak
 */
public class GeneratedArtifactsComposite extends Composite {
    /** The group object which surrender all this part. */
    private Group generateArtifactsGroup;
    /** The checkbox asking if defalt artifacts should be generated. */
    private Button generateDefaultArtifactsButton;
    /** The state of the checkbox button. Equivalent to know if the text should be enabled or not. */
    private boolean generateDefaultComponent = true;;
    /** 
     * The SWT Text displays the names of the generated artifacts.
     * In case where the "default..." is not checked, it means no file will be generated. 
     * Instead of that, the wizard simply writes the chosen name into the jbi.xml file.
     * The name depends on the version of the CDK chosen above in the wizard page.
     * 
     * @See 'NewComponentProjectWizardComposite' for the entire wizard page.
     */
    private Text artifactsVersionText; 
    
    /**
     * TODO: use XML files for this part.
     * TODO: create a default XML package managing access to XML files. 
     * TODO: ensure that CDK versions have the same name in the different files. 
     */
    private HashMap<String, String> BCArtifacts = new HashMap<String, String>();
    private HashMap<String, String> SEArtifacts = new HashMap<String, String>();
              
    
    /** Constructor. Called by the Eclipse bundle. */
    GeneratedArtifactsComposite( Composite parent, int style ) {
        super(parent, style);
        
        // TODO: see above about XML files...
        BCArtifacts.put( "2.0", "org.objectweb.petals.component.framework.bc.DefaultBindingComponent" ); //$NON-NLS-1$ //$NON-NLS-2$
        SEArtifacts.put( "2.0", "org.objectweb.petals.component.framework.se.DefaultServiceEngine" ); //$NON-NLS-1$ //$NON-NLS-2$
        
        initialize();   // define graphical elements
    }

    
    /**
     * Define the elements displayed on screen. 
     */
    private void initialize() {
        // first, define a layout on the composite
        this.setLayout( new GridLayout());
        
        // define the group container - the box surrounding the widget and its title
        generateArtifactsGroup = new Group( this, SWT.NONE );
        generateArtifactsGroup.setText( Messages.GeneratedArtifactsComposite_4 );
        generateArtifactsGroup.setLayout( new GridLayout());        
        
        // add a checkbox button: generate default artifacts or just fill in the jbi.xml ?
        generateDefaultArtifactsButton = new Button( generateArtifactsGroup, SWT.CHECK );
        generateDefaultArtifactsButton.setText( Messages.GeneratedArtifactsComposite_5 );
        generateDefaultArtifactsButton.setToolTipText( 
                Messages.GeneratedArtifactsComposite_6
                + Messages.GeneratedArtifactsComposite_7
        );
        
        // the checkbox must be checked at the beginning
        generateDefaultArtifactsButton.setSelection( generateDefaultComponent );
        
        /* add a listener on this button:
         * if its is checked, the SWT Text must be disabled. 
         * Otherwise, the SWT Text is enabled.
         */
        generateDefaultArtifactsButton.addSelectionListener( new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }

            public void widgetSelected(SelectionEvent e) {
                toggleSWTText();      // enable / disable SWT Text                
            }            
        });        
        
        // create the SWT Text and disable it
        artifactsVersionText = new Text( generateArtifactsGroup, SWT.READ_ONLY | SWT.BORDER | SWT.SINGLE );
        artifactsVersionText.setEnabled( false );
        
        // BC with cdk2.0 are chosen by default at the beginning
        artifactsVersionText.setText( BCArtifacts.get( "2.0" )); //$NON-NLS-1$
        artifactsVersionText.setToolTipText( Messages.GeneratedArtifactsComposite_9 );
    }
    
    
    /**
     * If the SWT Text is enabled, disable it. If it is disabled, enable it.
     * 
     * Pre-condition: the SWT Text exists.
     *      "artifactsVersionText != null && !artifactsVersionText.isDisposed()"
     */
    private void toggleSWTText() {
        // disable / enable SWT Text
        generateDefaultComponent = !generateDefaultComponent;
        artifactsVersionText.setEnabled( generateDefaultComponent );        
    }
    
    
    /**
     * Change the item displayed in the SWT Text. 
     * @param BCwasChecked true if 'BC' was chosen in the wizard. False if SE was chosen.
     * @param cdkVersion the version of the CDK.
     *
     * Pre-condition: the SWT Text exists.
     *      "artifactsVersionText != null && !artifactsVersionText.isDisposed()"
     * Pre-condition: the cdk version is valid.
     *      "cdkVersion != null && ..."
     */
    public void changeComponentType( boolean BCwasChecked, String cdkVersion ) {
        String text;
        if( BCwasChecked )
            text = BCArtifacts.get( cdkVersion );
        else
            text = SEArtifacts.get( cdkVersion );
        
        // set text - and ensure we don't set a null String
        if( text == null )
            text = ""; //$NON-NLS-1$
        artifactsVersionText.setText( text );
        
        // change the tool tip text
        artifactsVersionText.setToolTipText( Messages.GeneratedArtifactsComposite_11 
                + cdkVersion + Messages.GeneratedArtifactsComposite_12 );
    }
    

    /**
     * @return the generateDefaultComponent
     */
    public boolean isGenerateDefaultComponent() {
        return generateDefaultComponent;
    }
}
