package org.objectweb.petals.tools.plugin.monitor.model;

import java.io.IOException;
import java.lang.reflect.UndeclaredThrowableException;
import java.net.MalformedURLException;
import java.net.SocketException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.jbi.JBIException;
import javax.jbi.management.AdminServiceMBean;
import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.management.DeploymentServiceMBean;
import javax.jbi.management.InstallationServiceMBean;
import javax.jbi.management.InstallerMBean;
import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.IPropertySource;
import org.objectweb.petals.binding.servicesexplorer.ServicesExplorerMBean;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.nmr.Message;
import org.objectweb.petals.tools.plugin.monitor.nmr.MessagesCollector;
import org.objectweb.petals.tools.plugin.monitor.nmr.MessagesListener;
import org.objectweb.petals.tools.plugin.monitor.preferences.PreferenceConstants;
import org.objectweb.petals.tools.plugin.monitor.properties.PetalsServerPropertySource;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeObject;
import org.objectweb.petals.tools.plugin.monitor.views.MonitoringView;
import org.objectweb.petals.tools.plugin.monitor.views.PetalsServersView;


public class PetalsServer extends ComponentParent implements IAdaptable {

    private static final long serialVersionUID = 7954283394819743694L;
    private static final String JMX_DEFAULT_PREFIX = "service:jmx:";
    private static final String JMX_DEFAULT_PROTOCOL = "rmi:///jndi/rmi://";
    private static final String JMX_PATH = "/management/rmi-jmx-connector";
    private static final String MONITORING_MBEAN_NAME = "Petals:name=Monitoring,type=service";
    private static final String PETALS_ADMIN_MBEAN_NAME = "Petals:name=PetalsAdmin,type=service";
    
    public static final String CONF_HOST = "host";
    public static final String CONF_HTML_PORT = "htmlPort";
    public static final String CONF_JMX_LOGIN = "jmxLogin";
    public static final String CONF_JMX_PASSWORD = "jmxPassword";
    public static final String CONF_JMX_PORT = "jmxPort";
    public static final String CONF_JNDI_FACTORY = "jndiFactory";
    public static final String CONF_JNDI_PORT = "jndiPort";
    public static final String CONF_JORAM_DOMAIN = "joramDomain";
    public static final String CONF_JORAM_DOMAIN_PORT = "joramDomainPort";
    public static final String CONF_JORAM_ID = "joramId";
    public static final String CONF_JORAM_LOGIN = "joramLogin";
    public static final String CONF_JORAM_PASSWORD = "joramPassword";
    public static final String CONF_JORAM_TCP_PORT = "joramTCPPort";
    public static final String CONF_NAME = "name";
    public static final String CONF_STATE = "state";
    public static final String CONF_UID = "uid";
    
    private boolean connected;
    private boolean started;
    private boolean monitored;
    private String id;
    private String jmxUrl;
    private String jmxPort;
    private String jmxProtocol;
    private String jmxLogin;
    private String jmxPassword;
    private String host;
    private String httpPort;
    private String httpUrl;
    transient private JMXConnector connector;
    
    transient private MBeanServerConnection connection;
    
    transient private AdminServiceMBean admin;
    transient private InstallationServiceMBean install;
    transient private DeploymentServiceMBean deploy;
    transient private ServicesExplorerMBean nmr;
    
    transient private ObjectName petalsAdmin;
    transient private ObjectName monitoring;
    
    transient private MessagesListener NMRMessagesListener;
    transient private MessagesCollector messagesCollector;
        
    public final static String BC_TEXT = "Binding Components";
    public final static String SE_TEXT = "Service Engines";
    public final static String SL_TEXT = "Shared Librairies";
    public final static String SA_TEXT = "Service Assemblies";
    
    private List<BindingComponent> bindingComponents;
    private List<ComponentParent> folders;
    private List<ServiceAssembly> serviceAssemblies;
    private List<SharedLibrary> sharedLibraries;
    private List<ServiceEngine> serviceEngines;
    
    private FolderComponent folderBC;
    private FolderComponent folderSE;
    private FolderComponent folderSL;
    private FolderComponent folderSA;
    
    transient private Shell shell;
    transient private Preferences prefs = Activator.getDefault().getPluginPreferences();
    
    private PetalsServerPropertySource properties;
    
    private List<URL> installedUrls = new ArrayList<URL>();
    
    public void setShell(Shell shell) {
        this.shell = shell;
    }

    /**
     * Construit un serveur Petals &agrave; partir de son URL JMX
     * @param url URL JMX du serveur Petals
     */
    public PetalsServer(String url) {
        super(url);
        setJmxUrl(url);
        setServer(this);
    }
        
    /**
     * Construit un serveur Petals &agrave; partir des informations
     * de d&eacute;couvertes
     * @param map
     */
    public PetalsServer(Map<String,String> map) {
        super();
        host = map.get(CONF_HOST);
        jmxPort = map.get(CONF_JMX_PORT);
        jmxLogin = map.get(CONF_JMX_LOGIN);
        jmxPassword = map.get(CONF_JMX_PASSWORD);
        httpPort = map.get(CONF_HTML_PORT);
        name = map.get(CONF_NAME);
        id = map.get(CONF_UID);
        started = "started".equalsIgnoreCase(map.get(CONF_STATE));
        setServer(this);
    }
    
    /**
     * R&eacute;cup&egrave;re l'URL JMX d'administration de ce serveur
     * @return URL JMX
     */
    public String getJmxUrl() {
        if (jmxUrl == null) {
            if (host != null && jmxPort != null) {
                jmxUrl = JMX_DEFAULT_PREFIX + (jmxProtocol == null ? JMX_DEFAULT_PROTOCOL : jmxProtocol) + host + ":" + jmxPort + JMX_PATH;
            } else {
                throw new RuntimeException("BAD JMX ACCESS");
            }
        }
        return jmxUrl;
    }

    /**
     * D&eacute;finit l'URL JMX d'administration de ce serveur
     * @param url URL JMX d'administration de ce serveur
     */
    public void setJmxUrl(String url) {
        this.jmxUrl = url;
    }

    /**
     * V&eacute;rifie si ce client est connect&eacute; au 
     * serveur JMX distant
     * @return vrai si le serveur est connect&eacute;
     */
    public boolean isConnected() {
        return connected;
    }
        
    /**
     * Connecte ce serveur et r&eacute;cup&egrave;re les beans JMX d'administration
     */
    public void connect() {
            try {
                JMXServiceURL serviceURL = new JMXServiceURL(getJmxUrl());
                connector = JMXConnectorFactory.newJMXConnector(serviceURL, null);
                connector.connect();
                connection = connector.getMBeanServerConnection();
                
                petalsAdmin = new ObjectName(PETALS_ADMIN_MBEAN_NAME);

                admin = (AdminServiceMBean)MBeanServerInvocationHandler.newProxyInstance(connection, new ObjectName("Petals:name=Admin,type=service"), AdminServiceMBean.class, true);
                install = (InstallationServiceMBean)MBeanServerInvocationHandler.newProxyInstance(connection, new ObjectName("Petals:name=Installation,type=service"), InstallationServiceMBean.class, true);
                deploy = (DeploymentServiceMBean)MBeanServerInvocationHandler.newProxyInstance(connection, new ObjectName("Petals:name=Deployment,type=service"), DeploymentServiceMBean.class, true);
                nmr = (ServicesExplorerMBean)MBeanServerInvocationHandler.newProxyInstance(connection, new ObjectName("ServicesExplorerAgent:name=servicesexplorer"), ServicesExplorerMBean.class, true);
                
                /*
                MemoryMXBean memory = JMX.newMBeanProxy(connection, new ObjectName("java.lang:type=Memory"), MemoryMXBean.class, true);
                MemoryUsage mu =  memory.getHeapMemoryUsage();
                System.out.println("init: " + mu.getInit() +  " - used: " + mu.getUsed() + " - Max: " + mu.getMax());
                
                ThreadMXBean thread = JMX.newMBeanProxy(connection, new ObjectName("java.lang:type=Threading"), ThreadMXBean.class, true);
                System.out.println(thread.getThreadCount() + " threads are running");
                */
                
                if (description == null) {
                    description = admin.getSystemInfo();
                }
                
                for (ObjectName service : admin.getSystemServices()) {
                    System.out.println("admin service: " + service);
                }
                
                connected = true;
                
                // preferences: monitoring automatique des serveurs connectes
                setMonitoring(prefs.getBoolean(PreferenceConstants.AUTO_MONITOR_CONNECTED_SERVERS));                    
        } catch(Exception e) {
            MessageDialog.openError(shell, "Connexion JMX", "Impossible de connecter le service JMX avec l'URL '" + jmxUrl + "' (" + e.getMessage() + ")");
            e.printStackTrace();
            connected = false;
        }
    }
    
    /**
     * D&eacute;connecte le serveur Petals
     */
    public void disconnect() {
        try {
            if (connector != null) {
                connector.close();
            }
        } catch (IOException e) {
            MessageDialog.openError(null, "D&eacute;connexion JMX", "Impossible de fermer la connexion JMX depuis l'URL '" + jmxUrl + "' (" + e.getMessage() + ")");
        } finally {
            invalidateCache();
            connected = false;      
        }
    }
    
    /**
     * Invalide l'UI du client
     */
    void invalidateCache() {
        folders = null;
        bindingComponents = null;
        serviceEngines = null;
        serviceAssemblies = null;
        sharedLibraries = null;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.petals.tools.plugin.monitor.model.ComponentParent#toString()
     */
    public String toString(){
        if (name != null && !name.equals("")) {
                return name + (description != null && !description.equals("") ? " (" + description + (id != null ? " - " + id : "") + ")" : "");    
        } 
        return jmxUrl;
    }
    
    /**
     * Renvoie la liste des BC de ce serveur
     * @return liste des BC de ce serveur
     */
    public List<BindingComponent> getBindingComponents() {
        if (bindingComponents==null) {
            bindingComponents = new ArrayList<BindingComponent>();
            if (admin != null) {
                try {
                    for (ObjectName bcs: admin.getBindingComponents()) {
                        System.out.println(bcs + " keys: " + bcs.getKeyPropertyListString());
                        ComponentLifeCycleMBean lifecycle = (ComponentLifeCycleMBean)MBeanServerInvocationHandler.newProxyInstance(connection, bcs, ComponentLifeCycleMBean.class, true);
                        BindingComponent bc = new BindingComponent(bcs, lifecycle);
                        bc.setConnection(connection);
                        bc.setInstall(install);
                        bc.setDeploy(deploy);
                        bc.setServer(this);
                        bindingComponents.add(bc);
                    }
                } catch(UndeclaredThrowableException e) {
                    error("Connexion perdue","La connexion JMX au serveur Petals depuis l'URL " + getJmxUrl() + " a \u00e9t\u00e9 r\u00e9initialis\u00e9e", e);
                    disconnect();
                    PetalsServersView.refresh();                    
                }
            }
        }
        return bindingComponents;
    }

    /**
     * Renvoie la liste des SE de ce serveur
     * @return Liste des SE de ce serveur
     */
    public List<ServiceEngine> getEngineComponents() {
        if (serviceEngines == null) {
            serviceEngines = new ArrayList<ServiceEngine>();
            if (admin != null) {
                try {
                    for (ObjectName ses: admin.getEngineComponents()) {
                        ComponentLifeCycleMBean lifecycle = (ComponentLifeCycleMBean)MBeanServerInvocationHandler.newProxyInstance(connection, ses, ComponentLifeCycleMBean.class, true);
                        System.out.println(ses + "'s state: " + lifecycle.getCurrentState());
                        ServiceEngine se = new ServiceEngine(ses, lifecycle);
                        se.setConnection(connection);
                        se.setInstall(install);
                        se.setDeploy(deploy);
                        se.setServer(this);
                        serviceEngines.add(se);
                    }
                } catch(UndeclaredThrowableException e) {
                    error("Connexion perdue","La connexion JMX au serveur Petals depuis l'URL " + getJmxUrl() + " a \u00e9t\u00e9 r\u00e9initialis\u00e9e", e);
                    disconnect();
                    PetalsServersView.refresh();                    
                }
            }
        }
        return serviceEngines;
    }

    /**
     * Renvoie la liste des SA de ce serveur
     * @return liste des SA de ce serveur
     */
    public List<ServiceAssembly> getServiceAssemblies() {
        if (serviceAssemblies==null) {
            serviceAssemblies = new ArrayList<ServiceAssembly>();
            if (deploy != null) {
                try {
                    for (String sa: deploy.getDeployedServiceAssemblies()) {
                        ServiceAssembly nouveauSA = new ServiceAssembly(sa);
                        nouveauSA.setDeploy(deploy);
                        nouveauSA.setServer(this);
                        serviceAssemblies.add(nouveauSA);
                    }
                } catch (Exception e) {
                    MessageDialog.openError(null, "R\u00e9cup\u00e9ration des SA", "Impossible de r\u00e9cup\u00e9rer les SA de ce serveir (" + e.getMessage() + ")");
                }
            }
        }
        return serviceAssemblies;
    }

    /**
     * Renvoie la liste des SL de ce serveur
     * @return liste des SL de ce serveur
     */
    public List<SharedLibrary> getSharedLibrairies() {
        if (sharedLibraries==null) {
            sharedLibraries = new ArrayList<SharedLibrary>();
            if (install != null) {
                //FIXME: API JMX pour r\u00e9cup\u00e9rer les SL d\u00e9j\u00e0 install\u00e9es au d\u00e9marrage du serveur
                /*SharedLibrary fictif = new SharedLibrary("fictif");
                fictif.setInstall(install);
                fictif.setServer(this);
                sharedLibraries.add(fictif);*/
            }
        }
        return sharedLibraries;
    }
    
    @Override
    public TreeObject[] getChildren() {
        if (!isConnected()) {
            return new TreeObject[0];
        }       
        if (folders==null) {
            folders = new ArrayList<ComponentParent>(); 
            if (prefs.getBoolean(PreferenceConstants.SHOW_BC)) {
                folderBC = new FolderComponent(BC_TEXT, getBindingComponents(), install, connection, this);
                folders.add(folderBC);
            } 
            if (prefs.getBoolean(PreferenceConstants.SHOW_SE)) {
                folderSE = new FolderComponent(SE_TEXT, getEngineComponents(), install, connection, this);
                folders.add(folderSE);
            }
            if (prefs.getBoolean(PreferenceConstants.SHOW_SL)) {
                folderSL = new FolderComponent(SL_TEXT, getSharedLibrairies(), install, connection, this);
                folders.add(folderSL);
            }
            if (prefs.getBoolean(PreferenceConstants.SHOW_SA)) {
                folderSA = new FolderComponent(SA_TEXT, getServiceAssemblies(), install, connection, this);
                folders.add(folderSA);
            }
        } 
        return folders.toArray(new TreeObject[folders.size()]);
    }
    
    /**
     * Eteint ce serveur
     * @param stopTransportLayer pr\u00e9cise si la couche transport doit aussi \u00eatre \u00e9teinte
     */
    public void shutdown(boolean stopTransportLayer) {
        if (petalsAdmin != null) {
            try {
                if (stopTransportLayer) {
                    connection.invoke(petalsAdmin, "shutdownContainer", null, null);
                } else {
                    connection.invoke(petalsAdmin, "stopContainer", null, null);
                }
            } catch(SocketException se) { 
                info("Arr\u00eat serveur", "Serveur '" + getName() + " correctement arr\u00eat\u00e9");
            } catch (Exception e) {
                error("Arr\u00eat serveur", "Impossible d'arr\u00eater le serveur '" + getName() + "' correctement (" + e.getMessage() + ")", e);
            }
        }
    }
    
    /**
     * Log l'erreur avec son titre, son message et son objet exception
     * @param title Titre de l'erreur
     * @param message Description de l'erreur
     * @param e Object erreur
     */
    private void error(String title, String message, Exception e) {
        MessageDialog.openError(null, title, message);
        System.err.println(title + ": " + message);
        e.printStackTrace();
    }
    
    /**
     * Log un message \u00e0 titre informatif
     * @param title Titre de l'info
     * @param message Message de l'info
     */
    private void info(String title, String message) {
        MessageDialog.openInformation(null, title, message);
        System.out.println(title + ": " + message);
    }
    
    /**
     * V\u00e9rifie si ce serveur est d\u00e9marr\u00e9
     * @return vrai si ce serveur est d\u00e9marr\u00e9
     */
    public boolean isStarted() {
        return started;
    }

    /**
     * Renvoie l'URL HTTP pour administrateur ce serveur
     * en mode navigateur HTTP
     * @return URL HTTP
     */
    public String getHttpUrl() {
        if (httpUrl == null) {
            if (host == null || httpPort == null) {
                return null;
            }
            httpUrl = "http://" + host + ":" + httpPort;
        }
        return httpUrl;
    }

    /**
     * R\u00e9cup\u00e8re le login JMX de ce serveur
     * @return Login JMX du serveur
     */
    public String getJmxLogin() {
        return jmxLogin;
    }

    /**
     * R\u00e9cup\u00e8re le mot de passe JMX de ce serveur
     * @return Mot de passe JMX du serveur
     */
    public String getJmxPassword() {
        return jmxPassword;
    }

    /**
     * R\u00e9cup\u00e8re le port JMX de ce serveur
     * @return Port JMX de ce serveur
     */
    public String getJmxPort() {
        return jmxPort;
    }

    /**
     * R\u00e9cup\u00e8re le nom d'h\u00f4te JMX de ce serveur
     * @return Nom d'h\u00f4te JMX de ce serveur
     */
    public String getHost() {
        return host;
    }

    /**
     * Renvoie le protocole JMX sous forme textuelle
     * @return Protocole JMX
     */
    public String getJmxProtocol() {
        if (jmxProtocol == null) {
            return JMX_DEFAULT_PROTOCOL;
        }
        return jmxProtocol;
    }

    /** 
     * Renvoie les serveurs d\u00e9couverts sur le meme domaine
     * @return Liste des serveurs d\u00e9couverts sur le meme domaine
     */
    public Collection<Map<String,String>> getDiscoveredRemoteServers() {
        Collection<Map<String,String>> returnedCollectionMap = new ArrayList<Map<String,String>>();
        if (petalsAdmin != null) {
            try {
                if (prefs.getBoolean(PreferenceConstants.DISCOVER_ONLY_STARTED_SERVERS)) {
                    returnedCollectionMap = (Collection<Map<String,String>>) connection.getAttribute(petalsAdmin, "AllStartedServersConfiguration");
                } else {
                    returnedCollectionMap = (Collection<Map<String,String>>) connection.getAttribute(petalsAdmin, "AllServersConfiguration");
                }
            } catch (AttributeNotFoundException e) {
                e.printStackTrace();
            } catch (InstanceNotFoundException e) {
                e.printStackTrace();
            } catch (MBeanException e) {
                e.printStackTrace();
            } catch (ReflectionException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return returnedCollectionMap;
    }
    
    @Override
    public Object getAdapter(Class key) {
        if (key == IPropertySource.class) {
            if (properties == null) {
                properties = new PetalsServerPropertySource(this);
            }
            return properties;
        }
        return super.getAdapter(key);
    }
        
    /**
     * V&eacute;rifie si le server est monitor&eacute;
     * @return true si le server est monitor&eacute;
     */
    public boolean isMonitored() {
        return monitored;
    }
    
    /**
     * @param monitored the monitored to set
     */
    public void setMonitored(boolean monitored) {
        this.monitored = monitored;
    }
    
    /**
     * Subscribe / Unsubscribe to Monitoring informations
     * @param monitored true or false
     */
    public void setMonitoring(boolean monitored) {
        if (connection != null) {
            try {
                monitoring = new ObjectName(MONITORING_MBEAN_NAME);
                
                connection.invoke(monitoring,
                        "activateMonitoring", new Object[] {monitored},
                        new String[] {"boolean"});
                setMonitored(monitored);
                
                if (monitored) {
                    NMRMessagesListener = new MessagesListener(this);
                    connection.addNotificationListener(monitoring, NMRMessagesListener, null, null);
                    if (messagesCollector == null) {
                        messagesCollector = new MessagesCollector();
                    }
                } else {
                    connection.removeNotificationListener(monitoring, NMRMessagesListener);
                    NMRMessagesListener = null;
                }
            } catch (InstanceNotFoundException e) {
                monitoring = null;
                NMRMessagesListener = null;
            } catch (NullPointerException e) {
                monitoring = null;
                NMRMessagesListener = null;
            } catch (IOException e) {
                monitoring = null;
                NMRMessagesListener = null;
            } catch (ListenerNotFoundException e) {
                monitoring = null;
                NMRMessagesListener = null;
            } catch (MBeanException e) {
                monitoring = null;
                NMRMessagesListener = null;
            } catch (ReflectionException e) {
                monitoring = null;
                NMRMessagesListener = null;
            } catch (MalformedObjectNameException e) {
                monitoring = null;
                NMRMessagesListener = null;
            }
        }
    }
    
    /**
     * Set the interval time for NMR messages
     * 
     * @param time The time in ms
     */
    public void setMonitoringTimeSendLimit(int time) {
        if ((connection != null) 
                && (isMonitored())
                && monitoring != null) {
            try {
                connection.setAttribute(monitoring, new Attribute("TimeSendLimit", (Integer) time));
            } catch (InstanceNotFoundException e) {
                e.printStackTrace();
            } catch (AttributeNotFoundException e) {
                e.printStackTrace();
            } catch (InvalidAttributeValueException e) {
                e.printStackTrace();
            } catch (MBeanException e) {
                e.printStackTrace();
            } catch (ReflectionException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    /**
     * @return Returns the TimeSendLimit for the monitored server
     */
    public int getMonitoringTimeSendLimit() {
        int result = 10000;
        
        if ((connection != null) 
                && (isMonitored())
                && monitoring != null) {
            Object timeAttr;
            try {
                timeAttr = connection.getAttribute(monitoring, "TimeSendLimit");
                if (timeAttr instanceof Integer) {
                    result = ((Integer) timeAttr).intValue();
                }
            } catch (AttributeNotFoundException e) {
                e.printStackTrace();
            } catch (InstanceNotFoundException e) {
                e.printStackTrace();
            } catch (MBeanException e) {
                e.printStackTrace();
            } catch (ReflectionException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        
        return result;
    }
    
    /**
     * Set the threshold messages number for NMR messages
     * 
     * @param threshold The messages number threshold
     */
    public void setMonitoringMessagesThreshold(int threshold) {
        if ((connection != null) 
                && (isMonitored())
                && monitoring != null) {
            try {
                connection.setAttribute(monitoring, new Attribute("MessagesThreshold", (Integer) threshold));
            } catch (InstanceNotFoundException e) {
                e.printStackTrace();
            } catch (AttributeNotFoundException e) {
                e.printStackTrace();
            } catch (InvalidAttributeValueException e) {
                e.printStackTrace();
            } catch (MBeanException e) {
                e.printStackTrace();
            } catch (ReflectionException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    /**
     * @return Returns the messages number threshold
     */
    public int getMonitoringMessagesThreshold() {
        int result = 50;
        
        if ((connection != null) 
                && (isMonitored())
                && monitoring != null) {
            Object messagesThreshold;
            try {
                messagesThreshold = connection.getAttribute(monitoring, "MessagesThreshold");
                if (messagesThreshold instanceof Integer) {
                    result = ((Integer) messagesThreshold).intValue();
                }
            } catch (AttributeNotFoundException e) {
                e.printStackTrace();
            } catch (InstanceNotFoundException e) {
                e.printStackTrace();
            } catch (MBeanException e) {
                e.printStackTrace();
            } catch (ReflectionException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        
        return result;
    }

    /**
     * D&eacute;finit le login JMX pour administrateur ce serveur JMX
     * @param jmxLogin Login JMX
     */
    public void setJmxLogin(String jmxLogin) {
        this.jmxLogin = jmxLogin;
    }

    /**
     * D\u00e9finit le password JMX pour administrater ce serveur JMX
     * @param jmxPassword
     */
    public void setJmxPassword(String jmxPassword) {
        this.jmxPassword = jmxPassword;
    }

    @Override
    public String getType() {
        return "Serveur Petals";
    }
    
    /**
     * Teste si le PetalsServer contient le ServiceUnit pass\u00e9 en param\u00e8tre
     * 
     * @param su Le ServiceUnit \u00e0 tester
     * 
     * @return true si le PetalsServer contient le ServiceUnit
     */
    public boolean containsServiceUnit(ServiceUnit su) {
        boolean result = false;

        for (ServiceAssembly saCourant : serviceAssemblies) {
            for (ServiceUnit suCourant : saCourant.getSuList()) {
                if (suCourant == su) {
                    result = true;
                }
            }
        }

        return result;
    }
        
    /**
     * Installe un SE, BC ou SL dans ce serveur Petals \u00e0 partir de son URL
     * @param url
     * @return Composant install\u00e9, null si erreur
     */
    public ComponentParent install(URL url) {
        try {
            ComponentAnalyzer analyzer = new ComponentAnalyzer(url.toString());
            ComponentType type = analyzer.getType();
            switch(type) {
                case bindingComponent:
                case serviceEngine:
                    ObjectName newInstallerON = null; 
                        try {
                            newInstallerON = install.loadNewInstaller(url.toString());
                        } catch(Exception e) {
                            //TODO: en cas de non effacement du repo du comp de sa d\u00e9sinstallation
                            //r\u00e9cup\u00e9rer l'installer de cette d\u00e9sinstallation \u00e9chou\u00e9e et l'utiliser
                            //comme pt de chargement
                            //newInstallerON = install.loadInstaller(componentName)
                        }
                        if (newInstallerON == null) {
                            return null;
                        }
                        InstallerMBean installer =  (InstallerMBean)MBeanServerInvocationHandler.newProxyInstance(connection, newInstallerON, InstallerMBean.class, true);
                        if (!installer.isInstalled()) {
                            installedUrls.add(url);
                            ObjectName lifecycleON;
                            lifecycleON = installer.install();
                            ComponentLifeCycleMBean lifecycle = (ComponentLifeCycleMBean)MBeanServerInvocationHandler.newProxyInstance(connection, lifecycleON, ComponentLifeCycleMBean.class, true);
                            if (type == ComponentType.bindingComponent) {
                                BindingComponent bc = new BindingComponent(lifecycleON, lifecycle);
                                bc.setInstaller(installer);
                                bc.setInstall(install);
                                bc.setServer(this);
                                bc.setConnection(connection);
                                bc.setUrl(url.toString());
                                if (prefs.getBoolean(PreferenceConstants.START_INSTALLED_COMPONENTS) && !bc.isStarted()) {
                                    bc.start();
                                }
                                return bc;
                            } else if (type == ComponentType.serviceEngine) {
                                ServiceEngine se = new ServiceEngine(lifecycleON, lifecycle);
                                se.setInstaller(installer);
                                se.setInstall(install);
                                se.setServer(this);
                                se.setConnection(connection);
                                se.setUrl(url.toString());
                                if (prefs.getBoolean(PreferenceConstants.START_INSTALLED_COMPONENTS) && !se.isStarted()) {
                                    se.start();
                                }                               
                                return se;
                            }
                        }
                    break;
                case sharedLibrary:
                    installedUrls.add(url);
                    String slName = install.installSharedLibrary(url.toString());
                    if (slName == null) {
                        MessageDialog.openError(null, "Installation d'une biblioth\u00e8que partag\u00e9e", "Impossible de charger la librairie partag\u00e9e depuis l'URL '" + url + "' (elle est peut \u00eatre d\u00e9j\u00e0 install\u00e9e)");
                        return null;
                    } 
                    SharedLibrary sl = new SharedLibrary(slName);
                    sl.setInstall(install);
                    sl.setServer(this);
                    sl.setUrl(url.toString());
                    return sl;
                case serviceAssembly:
                    try {
                        String saReturnXmlValue = deploy.deploy(url.toString());
                        //System.out.println("assembly-return:[" + saReturnXmlValue + "]");                     
                        installedUrls.add(url);
                        ServiceAssembly sa = new ServiceAssembly(analyzer.getName());
                        sa.setReturnValue(saReturnXmlValue);
                        sa.setUrl(url.toString());
                        sa.setDeploy(deploy);
                        sa.setServer(this);
                        return sa;
                    } catch (Exception e) {
                        MessageDialog.openError(null, "D\u00e9ploiement d'un service assembly", "Impossible de d\u00e9ployer le service assembly point\u00e9 par l'URL '" + url + "' (" + e.getMessage() + ")");
                        return null;
                    }
                default:
                    MessageDialog.openError(null, "Installation d'un composant", "Impossible de d\u00e9terminer le composant point\u00e9 par l'URL '" + url + "'");
                    return null;
            }
        } catch(RuntimeException re) {
            MessageDialog.openError(null, "Installation d'un BC ou SE", "Le Zip accessible depuis l'URL '" + url + "' n'a pas pu \u00eatre valid\u00e9 (Le composant n'est pas reconnu ou est d\u00e9j\u00e0 install\u00e9)");
            re.printStackTrace();
        } catch (JBIException e) {
            MessageDialog.openError(null, "Installation d'un BC ou SE", "Impossible d'installer le composant depuis l'URL '" + url + "' (" + e.getMessage() + ")");
            e.printStackTrace();
        }
        return null;
    }   
        
    /**
     * Ajoute un SE ou un BC au sous mod\u00e8le du serveur
     * @param component
     */
    void addComponent(PetalsComponent component) {
        if (component instanceof BindingComponent) {
            folderBC.addChild(component);
            bindingComponents.add((BindingComponent)component);
        } else if (component instanceof ServiceEngine) {
            folderSE.addChild(component);
            serviceEngines.add((ServiceEngine)component);
        } 
    }
    
    /**
     * Ajoute une SL au sous mod\u00e8le du serveur
     * @param library
     */
    void addLibrary(SharedLibrary library) {
        folderSL.addChild(library);
    }
    
    /**
     * Supprime la SL du sous mod\u00e8le du serveur
     * @param library
     */
    void removeLibrary(SharedLibrary library) {
        folderSL.removeChild(library);
    }
    
    void removeAssembly(ServiceAssembly assembly) {
        folderSA.removeChild(assembly);
    }
    
    void removeComponent(PetalsComponent component) {
        if (component instanceof BindingComponent) {
            folderBC.removeChild(component);
        } else if (component instanceof ServiceEngine) {
            folderSE.removeChild(component);
        }
    }
    
    /**
     * Ajoute une SA au sous mod\u00e8le du serveur
     * @param assembly
     */
    void addAssembly(ServiceAssembly assembly) {
        folderSA.addChild(assembly);
    }
    
    /**
     * Ajoute une entit\u00e9 (SE, BC ou SL) au mod\u00e8le
     * @param componentParent entit\u00e9 \u00e0 ajouter
     */
    public void add(ComponentParent componentParent) {
        if (componentParent instanceof SharedLibrary) {
            SharedLibrary sl = (SharedLibrary)componentParent;
            addLibrary(sl);
            sharedLibraries.add(sl);
        } else if (componentParent instanceof ServiceAssembly) {
            ServiceAssembly sa = (ServiceAssembly)componentParent;
            addAssembly(sa);
            serviceAssemblies.add(sa);
        } else if (componentParent instanceof PetalsComponent) {
            addComponent((PetalsComponent)componentParent);
        }
    }

    /**
     * Supprime une entit\u00e9 (SE, BC ou SL) au mod\u00e8le
     * @param componentParent
     */
    public void remove(ComponentParent componentParent) {
        String url = componentParent.getUrl(); 
        if (url != null) {
            try {
                installedUrls.remove(new URL(url));
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }   
        }
        if (componentParent instanceof SharedLibrary) {
            removeLibrary((SharedLibrary)componentParent);
        } else if (componentParent instanceof ServiceAssembly) {
            removeAssembly((ServiceAssembly)componentParent);
        } else if (componentParent instanceof PetalsComponent) {
            removeComponent((PetalsComponent)componentParent);
        }
    }
    
    /**
     * V\u00e9rifie si le composant point\u00e9 par l'URL url 
     * a d\u00e9j\u00e0 \u00e9t\u00e9 install\u00e9
     * @param url URL du composant \u00e0 tester
     * @return vrai si le composant avec cette URL a d\u00e9j\u00e0 \u00e9t\u00e9 install\u00e9
     */
    public boolean isComponentUrlAlreadyInstalled(URL url) {
        return installedUrls.contains(url);
    }
    
    /**
     * V\u00e9rifie si un composant a d\u00e9j\u00e0 \u00e9t\u00e9 install\u00e9 sur ce serveur Petals
     * @param componentName nom du composant dont il faut v\u00e9rifier la pr\u00e9sence
     * @return vrai ssi le composant a d\u00e9j\u00e0 \u00e9t\u00e9 install\u00e9 sur ce serveur Petals
     */
    public boolean isComponentAlreadyInstalled(String componentName) {
        if (admin != null && componentName != null) {
            return admin.getComponentByName(componentName) != null; 
        }
        return false;
    }
    
    /**
     * D\u00e9charge tous les SE/BC, SL, et SA/SU de ce serveur PEtals
     */
    public void clearAllComponents() {
        List<BindingComponent> bcs = getBindingComponents();
        for (BindingComponent bc: bcs) {
            bc.uninstall();
            remove(bc);
        }
        List<ServiceEngine> ses = getEngineComponents();
        for (ServiceEngine se: ses) {
            se.uninstall();
            remove(se);
        }
        List<ServiceAssembly> sas = getServiceAssemblies();
        for (ServiceAssembly sa: sas) {
            try {
                sa.undeploy();
                remove(sa);
            } catch (Exception e) {
                MessageDialog.openError(null, "Nettoyage", "Impossible de nettoyer le SA " + sa);
            }
        }
        List<SharedLibrary> sls = getSharedLibrairies();
        for (SharedLibrary sl: sls) {
            sl.unload();
            remove(sl);
        }
        invalidateCache();
    }
    
    /**
     * Recharge tous les composants quand ceux-ci
     * sont rechargeables
     */
    public void updateAllComponents() {
        List<BindingComponent> bcs = getBindingComponents();
        for (BindingComponent bc: bcs) {
            bc.update();    
        }
        List<ServiceEngine> ses = getEngineComponents();
        for (ServiceEngine se: ses) {
            se.update();
        }
        List<ServiceAssembly> sas = getServiceAssemblies();
        for (ServiceAssembly sa: sas) {
            try {
                sa.redeploy();
            } catch (Exception e) {
                MessageDialog.openError(null, "Nettoyage", "Impossible de nettoyer le SA " + sa);
            }
        }
    }
    
    /**
     * V\u00e9rifie si le serveur contient au moins 1 BC ou 1 SE ou 1 SA ou 1 SL
     * @return
     */
    public boolean hasComponents() {
        return getBindingComponents().size() > 0 || getEngineComponents().size() > 0 || getSharedLibrairies().size() > 0 || getServiceAssemblies().size() > 0; 
    }
    
    /**
     * Taggue l'arbre des composants (BC, SA, SE, SL)
     * @param history
     * @param current
     */
    void tagTree(PetalsServer history) {
        tagComponent(history.getBindingComponents(), getBindingComponents());
        tagComponent(history.getEngineComponents(), getEngineComponents());
        tagComponent(history.getServiceAssemblies(), getServiceAssemblies());
        tagComponent(history.getSharedLibrairies(), getSharedLibrairies());
    }

    /**
     * Compare un composant en fonction de l'historique et appose
     * un nouveau flag issu du r\u00e9sultat de cette comparaison
     * @param oldComponents historique des composants
     * @param currentComponents composants courants
     */
    void tagComponent(List<? extends ComponentParent> oldComponents, List<? extends ComponentParent> currentComponents) {
        for (ComponentParent cp: oldComponents) {
            if (!containsComponent(currentComponents, cp)) {        
                // contains le BC a disparu depuis la derni\u00e8re connexion
                cp.setTag(ComponentParent.REMOVED_TAG);
                add(cp);
            }
        }
        for (ComponentParent cp: currentComponents) {
            if (!containsComponent(oldComponents,cp)) {
                // le BC n'existait pas \u00e0 la pr\u00e9c\u00e9dente connexion: il a \u00e9t\u00e9 ajout\u00e9
                cp.setTag(ComponentParent.ADDED_TAG);
            }
        }
    }
    
    /**
     * V\u00e9rifie si le composant appartient \u00e0 la liste
     * REMARQUE: 2 composants \u00e9gaux <=> avec le m\u00eame nom
     * REMARQUE: surcharge de equals() impossible pr effets de bord
     * @param componentsList
     * @param component
     * @return
     */
    boolean containsComponent(List<? extends ComponentParent> componentsList, ComponentParent component) {
        for (ComponentParent cp: componentsList) {
            if (cp.getName() != null && cp.getName().equals(component.getName())) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Supprime les tags des composants et 
     * supprime les composants fant\u00f4mes
     */
    public void refreshTags() {
        //FIXME: the same for SE/SA/SL
        clearTag(getBindingComponents());
        clearTag(getEngineComponents());
        clearTag(getServiceAssemblies());
        clearTag(getSharedLibrairies());        
    }
    
    void clearTag(List<? extends ComponentParent> components) {
        List<ComponentParent> componentsToDelete = new ArrayList<ComponentParent>();
        for (ComponentParent cp: components) {
            if (!cp.isNotTagged()) {
                if (cp.isRemovedTagged()) {
                    remove(cp);
                    componentsToDelete.add(cp);
                }
                cp.setTag(null);
            } 
        }
        components.removeAll(componentsToDelete);   
    }
    
    
    
    /**
     * D&eacute;clenche le taggage des composants
     */
    public void synchronize() {
        PetalsServer history = PetalsServersView.getInstance().getHistory(jmxUrl);
        if (history != null) {
            tagTree(history);
        }
    }
    
    public ServicesExplorerMBean getNMRBean() {
        return nmr;
    }

    public MBeanServerConnection getConnection() {
        return connection;
    }

    /**
     * @param messages The messages list to be added to the messagesCollector
     */
    public void collectMessages(List<Message> messages) {
        for (Message current : messages) {
            messagesCollector.addMessage(current);
        }
        
        if ((MonitoringView.getInstance() != null) 
                && MonitoringView.getInstance().getPetalsServer().equals(this)) {
            
            MonitoringView.getInstance().setPetalsServer(this);
        }
    }

    /**
     * @return the messagesCollector
     */
    public MessagesCollector getMessagesCollector() {
        return messagesCollector;
    }

    /**
     * @param messagesCollector the messagesCollector to set
     */
    public void setMessagesCollector(MessagesCollector messagesCollector) {
        this.messagesCollector = messagesCollector;
    }
    
}
